import { INotebookTracker } from '@jupyterlab/notebook';
import { isCodeCellModel } from '@jupyterlab/cells';
const CellMagicSyntaxMap = {
    ipython: 'text/x-ipython',
    '%%sql': 'text/x-pgsql',
    '%%sh': 'text/x-sh',
    '%%bash': 'text/x-sh',
    '%%html': 'text/html',
    '%%javascript': 'text/javascript',
    '%%js': 'text/javascript'
};
class SyntaxHighlighter {
    constructor(app, tracker) {
        this.app = app;
        this.tracker = tracker;
        // wait for JupyterLab page to startup/reload
        this.app.restored.then(() => {
            var _a, _b;
            // On Notebook loaded
            (_a = this.tracker.currentWidget) === null || _a === void 0 ? void 0 : _a.content.fullyRendered.connect(notebook => {
                // emit everytime cell rendered respectively
                if (this.tracker.currentWidget) {
                    const length = this.tracker.currentWidget.content.widgets.length;
                    this.setSyntax(this.tracker.currentWidget.content.widgets.slice(length - 1, length)[0]);
                }
            });
            // On current active cell content changed
            (_b = this.tracker.currentWidget) === null || _b === void 0 ? void 0 : _b.content.modelContentChanged.connect(() => {
                this.tracker.activeCell && this.setSyntax(this.tracker.activeCell);
            });
            // On current notebook change
            this.tracker.currentChanged.connect(tracker => {
                tracker.restored.then(() => {
                    var _a, _b, _c, _d;
                    // On notebook opened - cells added
                    (_b = (_a = tracker.currentWidget) === null || _a === void 0 ? void 0 : _a.model) === null || _b === void 0 ? void 0 : _b.cells.changed.connect((cellModels, change) => {
                        var _a;
                        // console.log(change.type, change.newIndex);
                        if (change.type === 'add') {
                            const widgets = (_a = tracker.currentWidget) === null || _a === void 0 ? void 0 : _a.content.widgets;
                            if (widgets) {
                                // console.log(widgets);
                                const cellWidget = widgets.find(widget => {
                                    return (widget.model.id === cellModels.get(change.newIndex).id);
                                });
                                // console.log(cellWidget);
                                cellWidget && this.setSyntax(cellWidget);
                            }
                        }
                    });
                    // On notebook model content changed - activeCell
                    (_d = (_c = tracker.currentWidget) === null || _c === void 0 ? void 0 : _c.model) === null || _d === void 0 ? void 0 : _d.contentChanged.connect(() => {
                        this.tracker.activeCell && this.setSyntax(this.tracker.activeCell);
                    });
                });
            });
        });
    }
    setSyntax(cell) {
        const editor = cell.editor
            .editor;
        if (cell !== null && isCodeCellModel(cell.model)) {
            const firstLine = editor.getDoc().getLine(0);
            const secondLine = editor.getDoc().getLine(1);
            const magic = firstLine.split(' ')[0];
            if (magic.startsWith('%%') && magic in CellMagicSyntaxMap) {
                // change to whatever defined in map
                this.highlight(editor, CellMagicSyntaxMap[magic]);
                return;
            }
            else if (firstLine.indexOf('spark.sql(') >= 0) {
                this.highlight(editor, CellMagicSyntaxMap['%%sql']);
            }
            else if (secondLine !== undefined) {
                if (secondLine.indexOf('spark.sql(') >= 0) {
                    this.highlight(editor, CellMagicSyntaxMap['%%sql']);
                }
            }
            else {
                // if not default then change to default
                this.highlight(editor, CellMagicSyntaxMap.ipython);
            }
        }
    }
    highlight(cellEditor, mode, retry = true) {
        const current_mode = cellEditor.getOption('mode');
        if (current_mode === 'null') {
            if (retry) {
                // putting at the end of execution queue to allow the CodeMirror mode to be updated
                // this will be invoked as soon as possible
                setTimeout(() => this.highlight(cellEditor, mode, false), 0);
            }
            return;
        }
        if (current_mode === mode) {
            return;
        }
        cellEditor.setOption('mode', mode);
    }
}
/**
 * Activate extension
 */
function activate(app, tracker) {
    console.log('JupyterLab extension jupyterlab-sparksql is activated!');
    new SyntaxHighlighter(app, tracker);
    // console.log('SyntaxHighlighter Loaded ', sh);
}
/**
 * Initialization data for the jupyterlab_sparksql extension.
 */
const plugin = {
    id: 'jupyterlab-sparksql:plugin',
    autoStart: true,
    requires: [INotebookTracker],
    activate: activate
};
export default plugin;
